import os
import requests
from gsc import auth


if os.getenv("GSC_ENV") == "dev":
    API_URL = "http://localhost:4000/api"
else:
    API_URL = "https://www.gitscientist.com/api"


class APIError(Exception):
    pass


def ping() -> str:
    cookies = auth.cookies()
    res = requests.get(API_URL + "/ping", cookies=cookies)
    if res.status_code == 200:
        return res.text
    raise api_error(res.status_code)


def complete_exercise(id: str) -> str:
    cookies = auth.cookies()
    res = requests.get(API_URL + "/complete-exercise/" + id, cookies=cookies)
    if res.status_code == 200:
        return res.text
    raise api_error(res.status_code)


def api_error(sc: int) -> APIError:
    if sc == 403:
        return APIError("Please log in.")
    elif sc == 404:
        return APIError("Not found.")
    elif sc == 500:
        return APIError("Server error.")
    return APIError(f"Request failed with code {sc}")



import subprocess
from gsc import cli


def clean_status() -> bool:
    res = subprocess.run(["git", "status"], capture_output=True)
    return b"working tree clean" in res.stdout


def on_branch(branch: str) -> bool:
    res = subprocess.run(["git", "branch"], capture_output=True)
    return b"* " + branch.encode("utf-8") in res.stdout


def tests_pass() -> bool:
    res = subprocess.run(["pytest"], capture_output=True)
    return res.returncode == 0


def check_commit_message(msg: str) -> None:
    cli.info(f"Checking commit message: {msg}")

    if msg.endswith("."):
        cli.warn(
            "You shouldn't put a full stop at the end of your Git commit messages.\n"
            "Messages should be concise and not gramatically correct sentences."
        )

    if len(msg) >= 50:
        cli.warn(
            "Git commit messages should be fewer than 50 characters long.\n"
            f"You've used {len(msg)}.\n"
            "Try to make your commit messages more concise."
        )



import os
import json
import pathlib
import requests
import http.server
import urllib.parse
import typing as t

from gsc import cli


TIMEOUT_S = 30 * 60
LOCAL_PORT = 4004
HOME = str(pathlib.Path.home())
GSC_DIR = HOME + "/.gitscientist/"
GSC_TOKEN = GSC_DIR + ".gsc_token"

if os.getenv("GSC_ENV") == "dev":
    BASE_URL = "http://localhost:4000"
else:
    BASE_URL = "https://www.gitscientist.com"
API_URL = BASE_URL + "/api"


class NoUserError(Exception):
    pass


class AuthenticationError(Exception):
    pass


class TokenHandler(http.server.BaseHTTPRequestHandler):
    def do_GET(self):
        parsed_path = urllib.parse.urlparse(self.path)

        res = requests.get(API_URL + parsed_path.path)

        self.send_response(302)

        if not os.path.exists(GSC_DIR):
            os.makedirs(GSC_DIR)

        if res.status_code == 200:
            session_unique_id = json.loads(res.text)["session_unique_id"]

            with open(GSC_TOKEN, "w") as f:
                f.write(session_unique_id)

            self.send_header("Location", BASE_URL + "/login-success")
        else:
            try:
                os.remove(GSC_TOKEN)
            except OSError:
                pass

            self.send_header("Location", BASE_URL + "/login-failure")

        self.end_headers()

    def log_message(self, format, *args):
        return


def login(email: str):
    res = requests.post(API_URL + "/login", data={"email": email})

    if res.status_code != 200:
        raise AuthenticationError("User not found.")

    cli.info("We sent you an email. Click the link inside to login.")

    # Start a localhost HTTP server to wait for the email verification token.
    server = http.server.HTTPServer(("", LOCAL_PORT), TokenHandler)
    server.handle_request()

    # Check for new token to see if we logged in successfully
    try:
        get_token()
    except AuthenticationError:
        raise AuthenticationError("Login Failed.")


def get_token() -> str:
    try:
        return open(GSC_TOKEN, "r").read()
    except FileNotFoundError:
        raise AuthenticationError("Not logged in.")


def cookies() -> t.Dict[str, str]:
    token = get_token()
    return dict(session_unique_id=token)



import os
from gsc import cli, client
import gsc.exercises.my_first_commit


class VerifyError(Exception):
    pass


def verify():
    while not os.path.exists(".git"):
        os.chdir("..")
        if os.getcwd() == "/":
            raise VerifyError("This is not a git repo.")

    if not os.path.exists(".gsc_id"):
        raise VerifyError("This repo is not a Git Scientist exercise.")

    gsc_id = open(".gsc_id", "r").read().strip()

    cli.title(f"Verifying {gsc_id}")

    if gsc_id == "my_first_commit":
        gsc.exercises.my_first_commit.verify()
        client.complete_exercise(gsc_id)
    else:
        raise VerifyError("Unknown Git Scientist exercise. Try updating gsc.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gsc', 'gsc.exercises']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.23.0,<3.0.0', 'typer>=0.1.1,<0.2.0']

entry_points = \
{'console_scripts': ['gsc = gsc.cli:main']}

setup_kwargs = {
    'name': 'gsc',
    'version': '0.0.2',
    'description': 'Git for Scientists practical exercise helper.',
    'long_description': '# gsc\n\nGit for Scientists practical exercise helper.\n\nSee https://www.gitscientist.com for more.\n',
    'author': 'Daniel Tipping',
    'author_email': 'daniel@oldreliable.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://www.gitscientist.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import subprocess

from gsc import verifier, cli
from gsc.exercises import utils


def verify():
    if not utils.clean_status():
        raise verifier.VerifyError(
            "Your git status is not clean. Run `git status` to see the problem."
        )

    if not utils.on_branch("master"):
        raise verifier.VerifyError(
            "You should be on the master branch for this exercise.\n"
            "Change branch with `git checkout master` and try again."
        )

    # We should be one commit ahead of remote master
    res = subprocess.run(
        ["git", "rev-list", "--left-right", "--count", "master...origin/master"],
        capture_output=True,
    )
    res = res.stdout.decode("utf-8").strip().split("\t")
    if ["1", "0"] != res:
        raise verifier.VerifyError(
            f"You need to make 1 commit but you've made {res[0]}."
        )

    # We should ideally have this exact commit message but no bother if not.
    res = subprocess.run(["git", "log", "-1", "--pretty=%B"], capture_output=True)
    res = res.stdout.decode("utf-8").strip()
    utils.check_commit_message(res)
    if res != "Switch `subtract` to use correct operator":
        cli.warn(
            "Looks like you didn't choose the suggested commit message.\n"
            'When you get to the "Useful Commit Messages" section, be sure to come back here '
            "and check that your message is good."
        )
    cli.success("Looks good.")



import typer
import gsc
from gsc import auth, client, verifier

app = typer.Typer()


def success(msg: str):
    typer.secho(msg, fg=typer.colors.GREEN, bold=True)


def info(msg: str):
    typer.echo(msg)


def title(msg: str):
    typer.secho(msg + "\n", fg=typer.colors.GREEN, bold=True)


def warn(msg: str):
    typer.secho(msg + "\n", fg=typer.colors.YELLOW, bold=False)


def error(msg: str):
    typer.secho(msg, fg=typer.colors.RED, bold=True)
    raise typer.Exit(1)


@app.command()
def login(email: str = typer.Option(..., prompt=True)):
    try:
        auth.login(email)
        success("Login Success.")

    except auth.AuthenticationError as e:
        error(str(e))


@app.command()
def ping():
    try:
        client.ping()
        success("Pong.")
    except auth.AuthenticationError as e:
        error(str(e))
    except client.APIError as e:
        error(str(e))


@app.command()
def verify():
    try:
        client.ping()
        verifier.verify()
        success("Exercise complete!")
    except verifier.VerifyError as e:
        error(str(e))
    except auth.AuthenticationError as e:
        error(str(e))
    except client.APIError as e:
        error(str(e))


@app.callback()
def version_callback(value: bool):
    if value:
        version()


@app.command(hidden=True)
def version():
    typer.echo(f"gsc version: {gsc.__version__}")
    raise typer.Exit()


@app.callback()
def options(version: bool = typer.Option(None, "--version", callback=version_callback)):
    """
    gsc is the Git for Scientists practical exercise helper.

    See https://www.gitscientist.com for more.
    """


def main():
    app()
